
#  jdtt_to_tdb
#
#    Return JD_TDB from JD_TT in TAI units

#    Transform the arrival time of a photon measured by an observer's TAI clock
#      on Earth's surface to represent a clock at the geocenter affected by
#      the presence of the Sun and other bodies in the solar system. This
#      is termed the Einstein delay at the geocenter.
#
#    Papers by Eastman, Markwardt, and Fairhead describe a periodic variation
#      with an amplitude of 3.4 milliseconds and dominant period of 1 year.
#      TDB converges on the JPL ephemeris time TEPH, the coordinate time CT
#      in the JPL ephemerides (Kaplan, 2005; Eastman, et al, 2010).
#
#    ERFA code derived from SOFA is a source for the Fairhead parameters
#      which in full form were not published in the 1990 paper.  Here we
#      also include a much faster and less accurate version from Kaplan
#      which is sufficiently precise for photometry in the regime
#      of microseconds, while the Fairhead model is accurate to nanoseconds.

#    The correction from UTC to TDB is 
#
#      Delta_Clock = N_Leap + 32.184 + (TDB - TT) seconds
#
#    This routine provides the third term of that expression.

#    Uses BigFloat data type for unlimited precision

#    Explanations: 
#      George H. Kaplan, USNO Circular 179, 2005
#      C. Markwardt, online 2001, retrieved December, 2021
#      L. Fairhead, P. Bretagnon, A&A 229, 240 (1990)
#      J. Eastman, R. Siverd, B.Scott Gaudi, PASP 122, 935 (2010)

#    Credit and comment:
#      The expansion constants in this  code come directly from
#      tdb2tdt written by Craig B. Markwardt and incorporated into 
#      Exofastv2 by Jason Eastman. The same parameters appear in 
#      the ERFA code as of 2022-01-09.  These 780 coefficients are said
#      in the Fairhead paper to provide an accuracy at the nanosecond
#      level, while the paper provides a subset with microsecond accuracy.

#    Validity: 
#      Expected to be valid for a few thousand years around J2000.0 to
#      accuracy of ~1 ns if using the Fairhead model, ~1 microsec if using
#      Kaplan model.

#    Explicit input:
#      JD_TT

#    Explicit output:
#      JD_TDB

#    Required data:
#      None

#    John Kielkopf (kielkopf@louisville.edu)
#    MIT License
#    Copyright 2022
#    
#    2022-01-22 Version 1.0



using Printf

#  Use true to select lower precision with faster execution 

kaplan_flag = true

#  Read and parse the command line arguments 
#  Return BigFloat array

function get_jdtt_args(args)
  nargs = length(args)
  
  if nargs < 1
    print("Enter space delimited JD_TTs on the command line \n")
    exit()
  end
 
  jd_tts = zeros(BigFloat,nargs)
  for i in 1:nargs
    jd_tts[i] = parse(BigFloat,args[i])
  end

  return jd_tts
end




#  Apply the delay to obtain TDB scale using Fairhead parameters

function include_fairhead_delay(jd_tts)

  fh_a = [        
  1656.674564e-6, 
    22.417471e-6,   
    13.839792e-6,   
     4.770086e-6,   
     4.676740e-6,   
     2.256707e-6,   
     1.694205e-6,   
     1.554905e-6,   
     1.276839e-6,   
     1.193379e-6,   
     1.115322e-6,   
     0.794185e-6,   
     0.447061e-6,   
     0.435206e-6,   
     0.600309e-6,   
     0.496817e-6,   
     0.486306e-6,   
     0.432392e-6,   
     0.468597e-6,   
     0.375510e-6,   
     0.243085e-6,   
     0.173435e-6,   
     0.230685e-6,   
     0.203747e-6,   
     0.143935e-6,   
     0.159080e-6,   
     0.119979e-6,   
     0.118971e-6,   
     0.116120e-6,   
     0.137927e-6,   
     0.098358e-6,   
     0.101868e-6,   
     0.080164e-6,   
     0.079645e-6,   
     0.062617e-6,   
     0.075019e-6,   
     0.064397e-6,   
     0.063814e-6,   
     0.048042e-6,   
     0.048373e-6,   
     0.058844e-6,   
     0.046551e-6,   
     0.054139e-6,   
     0.042411e-6,   
     0.040184e-6,   
     0.036564e-6,   
     0.040759e-6,   
     0.036507e-6,   
     0.036955e-6,   
     0.042732e-6,   
     0.042560e-6,   
     0.040480e-6,   
     0.028244e-6,   
     0.033477e-6,   
     0.034867e-6,   
     0.032438e-6,   
     0.030215e-6,   
     0.029247e-6,   
     0.033529e-6,   
     0.032423e-6,   
     0.027567e-6,   
     0.029862e-6,   
     0.022509e-6,   
     0.020937e-6,   
     0.020322e-6,   
     0.024816e-6,   
     0.025196e-6,   
     0.021691e-6,   
     0.017673e-6,   
     0.022567e-6,   
     0.016155e-6,   
     0.014751e-6,   
     0.015949e-6,   
     0.015974e-6,   
     0.014223e-6,   
     0.017806e-6,   
     0.013671e-6,   
     0.011942e-6,   
     0.014318e-6,   
     0.012462e-6,   
     0.010962e-6,   
     0.015078e-6,   
     0.010396e-6,   
     0.011707e-6,   
     0.010453e-6,   
     0.012420e-6,   
     0.011847e-6,   
     0.008610e-6,   
     0.011622e-6,   
     0.010825e-6,   
     0.008666e-6,   
     0.009963e-6,   
     0.009858e-6,   
     0.007959e-6,   
     0.010099e-6,   
     0.007147e-6,   
     0.007505e-6,   
     0.008323e-6,   
     0.007490e-6,   
     0.009370e-6,   
     0.007117e-6,   
     0.007857e-6,   
     0.007019e-6,   
     0.006056e-6,   
     0.008107e-6,   
     0.006731e-6,   
     0.007332e-6,   
     0.006366e-6,   
     0.006858e-6,   
     0.006919e-6,   
     0.006826e-6,   
     0.005308e-6,   
     0.005096e-6,   
     0.004841e-6,   
     0.005582e-6,   
     0.006304e-6,   
     0.006603e-6,   
     0.005123e-6,   
     0.004648e-6,   
     0.005119e-6,   
     0.004521e-6,   
     0.005680e-6,   
     0.005488e-6,   
     0.004193e-6,   
     0.003742e-6,   
     0.004148e-6,   
     0.004553e-6,   
     0.004892e-6,   
     0.004044e-6,   
     0.004164e-6,   
     0.004349e-6,   
     0.003919e-6,   
     0.003129e-6,   
     0.004080e-6,   
     0.003270e-6,   
     0.002954e-6,   
     0.002872e-6,   
     0.002881e-6,   
     0.003279e-6,   
     0.003625e-6,   
     0.003074e-6,   
     0.002775e-6,   
     0.002646e-6,   
     0.002575e-6,   
     0.003500e-6,   
     0.002740e-6,   
     0.002464e-6,   
     0.002409e-6,   
     0.003354e-6,   
     0.002296e-6,   
     0.003002e-6,   
     0.003202e-6,   
     0.002954e-6,   
     0.002353e-6,   
     0.002401e-6,   
     0.003053e-6,   
     0.003024e-6,   
     0.002863e-6,   
     0.002103e-6,   
     0.002303e-6,   
     0.002303e-6,   
     0.002381e-6,   
     0.002493e-6,   
     0.002366e-6,   
     0.002169e-6,   
     0.002397e-6,   
     0.002183e-6,   
     0.002353e-6,   
     0.002199e-6,   
     0.001729e-6,   
     0.001896e-6,   
     0.002085e-6,   
     0.002024e-6,   
     0.001737e-6,   
     0.002229e-6,   
     0.001602e-6,   
     0.002186e-6,   
     0.001897e-6,   
     0.001825e-6,   
     0.001894e-6,   
     0.001421e-6,   
     0.001408e-6,   
     0.001847e-6,   
     0.001391e-6,   
     0.001388e-6,   
     0.001810e-6,   
     0.001288e-6,   
     0.001297e-6,   
     0.001335e-6,   
     0.001376e-6,   
     0.001745e-6,   
     0.001649e-6,   
     0.001416e-6,   
     0.001238e-6,   
     0.001472e-6,   
     0.001169e-6,   
     0.001039e-6,   
     0.001004e-6,   
     0.001284e-6,   
     0.001278e-6,   
     0.001321e-6,   
     0.001297e-6,   
     0.000954e-6,   
     0.001145e-6,   
     0.000979e-6,   
     0.000987e-6,   
     0.001070e-6,   
     0.000991e-6,   
     0.001155e-6,   
     0.001176e-6,   
     0.000890e-6,   
     0.000884e-6,   
     0.000876e-6,   
     0.000806e-6,   
     0.000773e-6,   
     0.001077e-6,   
     0.000954e-6,   
     0.000737e-6,   
     0.000845e-6,   
     0.000819e-6,   
     0.000852e-6,   
     0.000723e-6,   
     0.000940e-6,   
     0.000885e-6,   
     0.000706e-6,   
     0.000732e-6,   
     0.000764e-6,   
     0.000908e-6,   
     0.000907e-6,   
     0.000673e-6,   
     0.000814e-6,   
     0.000630e-6,   
     0.000798e-6,   
     0.000798e-6,   
     0.000806e-6,   
     0.000607e-6,   
     0.000601e-6,   
     0.000646e-6,   
     0.000704e-6,   
     0.000603e-6,   
     0.000609e-6,   
     0.000631e-6,   
     0.000576e-6,   
     0.000674e-6,   
     0.000726e-6,   
     0.000710e-6,   
     0.000647e-6,   
     0.000678e-6,   
     0.000618e-6,   
     0.000738e-6,   
     0.000660e-6,   
     0.000694e-6,   
     0.000531e-6,   
     0.000611e-6,   
     0.000575e-6,   
     0.000553e-6,   
     0.000689e-6,   
     0.000495e-6,   
     0.000567e-6,   
     0.000515e-6,   
     0.000486e-6,   
     0.000662e-6,   
     0.000509e-6,   
     0.000472e-6,   
     0.000461e-6,   
     0.000641e-6,   
     0.000520e-6,   
     0.000493e-6,   
     0.000478e-6,   
     0.000472e-6,   
     0.000559e-6,   
     0.000494e-6,   
     0.000463e-6,   
     0.000432e-6,   
     0.000574e-6,   
     0.000484e-6,   
     0.000550e-6,   
     0.000399e-6,   
     0.000491e-6,   
     0.000432e-6,   
     0.000481e-6,   
     0.000480e-6,   
     0.000485e-6,   
     0.000426e-6,   
     0.000480e-6,   
     0.000466e-6,   
     0.000520e-6,   
     0.000458e-6,   
     0.000470e-6,   
     0.000416e-6,   
     0.000449e-6,   
     0.000465e-6,   
     0.000362e-6,   
     0.000383e-6,   
     0.000389e-6,   
     0.000331e-6,   
     0.000430e-6,   
     0.000368e-6,   
     0.000330e-6,   
     0.000332e-6,   
     0.000384e-6,   
     0.000387e-6,   
     0.000325e-6,   
     0.000318e-6,   
     0.000305e-6,   
     0.000352e-6,   
     0.000311e-6,   
     0.000297e-6,   
     0.000363e-6,   
     0.000323e-6,   
     0.000341e-6,   
     0.000290e-6,   
     0.000342e-6,   
     0.000329e-6,   
     0.000374e-6,   
     0.000285e-6,   
     0.000338e-6,   
     0.000276e-6,   
     0.000336e-6,   
     0.000290e-6,   
     0.000318e-6,   
     0.000271e-6,   
     0.000331e-6,   
     0.000292e-6,   
     0.000362e-6,   
     0.000280e-6,   
     0.000267e-6,   
     0.000262e-6,   
     0.000250e-6,   
     0.000325e-6,   
     0.000268e-6,   
     0.000284e-6,   
     0.000301e-6,   
     0.000294e-6,   
     0.000236e-6,   
     0.000234e-6,   
     0.000268e-6,   
     0.000265e-6,   
     0.000280e-6,   
     0.000292e-6,   
     0.000223e-6,   
     0.000301e-6,   
     0.000264e-6,   
     0.000304e-6,   
     0.000301e-6,   
     0.000260e-6,   
     0.000299e-6,   
     0.000211e-6,   
     0.000209e-6,   
     0.000240e-6,   
     0.000216e-6,   
     0.000203e-6,   
     0.000200e-6,   
     0.000197e-6,   
     0.000227e-6,   
     0.000197e-6,   
     0.000205e-6,   
     0.000209e-6,   
     0.000208e-6,   
     0.000191e-6,   
     0.000190e-6,   
     0.000264e-6,   
     0.000256e-6,   
     0.000188e-6,   
     0.000185e-6,   
     0.000198e-6,   
     0.000195e-6,   
     0.000234e-6,   
     0.000188e-6,   
     0.000222e-6,   
     0.000181e-6,   
     0.000171e-6,   
     0.000206e-6,   
     0.000169e-6,   
     0.000191e-6,   
     0.000228e-6,   
     0.000184e-6,   
     0.000220e-6,   
     0.000166e-6,   
     0.000191e-6,   
     0.000180e-6,   
     0.000163e-6,   
     0.000225e-6,   
     0.000222e-6,   
     0.000204e-6,   
     0.000159e-6,   
     0.000200e-6,   
     0.000187e-6,   
     0.000161e-6,   
     0.000205e-6,   
     0.000189e-6,   
     0.000168e-6,   
     0.000149e-6,   
     0.000189e-6,   
     0.000143e-6,   
     0.000146e-6,   
     0.000144e-6,   
     0.000175e-6,   
     0.000162e-6,   
     0.000187e-6,   
     0.000146e-6,   
     0.000180e-6,   
     0.000148e-6,   
     0.000157e-6,   
     0.000167e-6,   
     0.000133e-6,   
     0.000154e-6,   
     0.000148e-6,   
     0.000128e-6,   
     0.000130e-6,   
     0.000152e-6,   
     0.000138e-6,   
     0.000123e-6,   
     0.000140e-6,   
     0.000126e-6,   
     0.000119e-6,   
     0.000151e-6,   
     0.000117e-6,   
     0.000165e-6,   
     0.000117e-6,   
     0.000130e-6,   
     0.000121e-6,   
     0.000162e-6,   
     0.000141e-6,   
     0.000118e-6,   
     0.000129e-6,   
     0.000126e-6,   
     0.000114e-6,   
     0.000120e-6,   
     0.000115e-6,   
     0.000126e-6,   
     0.000158e-6,   
     0.000134e-6,   
     0.000151e-6,   
     0.000109e-6,   
     0.000131e-6,   
     0.000146e-6,   
     0.000146e-6,   
     0.000107e-6,   
     0.000138e-6,   
     0.000113e-6,   
     0.000115e-6,   
     0.000138e-6,   
     0.000139e-6,   
     0.000146e-6,   
     0.000107e-6,   
     0.000142e-6,   
     0.000128e-6,   
     0.000135e-6,   
     0.000101e-6,   
     0.000104e-6,   
     0.000103e-6,   
     0.000119e-6,   
     0.000138e-6,   
     0.000121e-6,   
     0.000123e-6,   
     0.000119e-6,   
     0.000133e-6,   
     0.000129e-6,   
     0.000131e-6,   
     0.000104e-6,   
     0.000112e-6,   
     0.000123e-6,   
     0.000121e-6,   
     0.000108e-6,   
     0.000113e-6,   
     0.000109e-6,   
     0.000101e-6,   
     0.000113e-6,   
     0.000113e-6,   
     0.000106e-6,   
     0.000101e-6,   
     0.000103e-6,   
     0.000101e-6]   
                             
  fh_omega_a = [             
     6283.075849991,         
     5753.384884897,         
    12566.151699983,         
      529.690965095,         
     6069.776754553,         
      213.299095438,         
      -3.523118349,          
    77713.771467920,         
     7860.419392439,         
     5223.693919802,         
     3930.209696220,         
    11506.769769794,         
       26.298319800,         
     -398.149003408,         
     1577.343542448,         
     6208.294251424,         
     5884.926846583,         
       74.781598567,         
     6244.942814354,         
     5507.553238667,         
     -775.522611324,         
    18849.227549974,         
     5856.477659115,         
    12036.460734888,         
     -796.298006816,         
    10977.078804699,         
       38.133035638,         
     5486.777843175,         
     1059.381930189,         
    11790.629088659,         
     2544.314419883,         
    -5573.142801634,         
      206.185548437,         
     4694.002954708,         
       20.775395492,         
     2942.463423292,         
     5746.271337896,         
     5760.498431898,         
     2146.165416475,         
      155.420399434,         
      426.598190876,         
       -0.980321068,         
    17260.154654690,         
     6275.962302991,         
       -7.113547001,         
     5088.628839767,         
    12352.852604545,         
      801.820931124,         
     3154.687084896,         
      632.783739313,         
   161000.685737473,         
    15720.838784878,         
    -6286.598968340,         
     6062.663207553,         
      522.577418094,         
     6076.890301554,         
     7084.896781115,         
   -71430.695617928,         
     9437.762934887,         
     8827.390269875,         
     6279.552731642,         
    12139.553509107,         
    10447.387839604,         
     8429.241266467,         
      419.484643875,         
    -1194.447010225,         
     1748.016413067,         
    14143.495242431,         
     6812.766815086,         
     6133.512652857,         
    10213.285546211,         
     1349.867409659,         
     -220.412642439,         
    -2352.866153772,         
    17789.845619785,         
       73.297125859,         
     -536.804512095,         
     8031.092263058,         
    16730.463689596,         
      103.092774219,         
        3.590428652,         
    19651.048481098,         
      951.718406251,         
    -4705.732307544,         
     5863.591206116,         
     4690.479836359,         
     5643.178563677,         
     3340.612426700,         
     5120.601145584,         
      553.569402842,         
     -135.065080035,         
      149.563197135,         
     6309.374169791,         
      316.391869657,         
      283.859318865,         
     -242.728603974,         
     5230.807466803,         
    11769.853693166,         
    -6256.777530192,         
   149854.400134205,         
       38.027672636,         
    12168.002696575,         
     6206.809778716,         
      955.599741609,         
    13367.972631107,         
     5650.292110678,         
       36.648562930,         
     4164.311989613,         
     5216.580372801,         
     6681.224853400,         
     7632.943259650,         
    -1592.596013633,         
    11371.704689758,         
     5333.900241022,         
     5966.683980335,         
    11926.254413669,         
    23581.258177318,         
       -1.484472708,         
     1589.072895284,         
     6438.496249426,         
     4292.330832950,         
    23013.539539587,         
       -3.455808046,         
     7234.794256242,         
     7238.675591600,         
     -110.206321219,         
    11499.656222793,         
     5436.993015240,         
     4732.030627343,         
    12491.370101415,         
    11513.883316794,         
    12528.018664345,         
     6836.645252834,         
    -7058.598461315,         
       76.266071276,         
     6283.143160294,         
       28.449187468,         
      735.876513532,         
     5849.364112115,         
     6209.778724132,         
      949.175608970,         
     9917.696874510,         
    10973.555686350,         
    25132.303399966,         
      263.083923373,         
    18319.536584880,         
      202.253395174,         
        2.542797281,         
   -90955.551694697,         
     6496.374945429,         
     6172.869528772,         
    27511.467873537,         
    -6283.008539689,         
      639.897286314,         
    16200.772724501,         
   233141.314403759,         
    83286.914269554,         
    17298.182327326,         
    -7079.373856808,         
    83996.847317911,         
    18073.704938650,         
       63.735898303,         
     6386.168624210,         
        3.932153263,         
    11015.106477335,         
     6243.458341645,         
     1162.474704408,         
     6246.427287062,         
     -245.831646229,         
     3894.181829542,         
    -3128.388765096,         
       35.164090221,         
    14712.317116458,         
     6290.189396992,         
      491.557929457,         
    14314.168113050,         
      454.909366527,         
    22483.848574493,         
    -3738.761430108,         
     1052.268383188,         
       20.355319399,         
    10984.192351700,         
    10873.986030480,         
    -8635.942003763,         
       -7.046236698,         
   -88860.057071188,         
    -1990.745017041,         
    23543.230504682,         
     -266.607041722,         
    10969.965257698,         
   244287.600007027,         
    31441.677569757,         
     9225.539273283,         
     4804.209275927,         
     4590.910180489,         
     6040.347246017,         
     5540.085789459,         
     -170.672870619,         
    10575.406682942,         
       71.812653151,         
    18209.330263660,         
    21228.392023546,         
     6282.095528923,         
     6058.731054289,         
     5547.199336460,         
    -6262.300454499,         
  -154717.609887482,         
     4701.116501708,         
      -14.227094002,         
      277.034993741,         
    13916.019109642,         
    -1551.045222648,         
     5017.508371365,         
    15110.466119866,         
    -4136.910433516,         
      175.166059800,         
    -6284.056171060,         
     5326.786694021,         
     -433.711737877,         
     8662.240323563,         
      199.072001436,         
    17256.631536341,         
     6037.244203762,         
    11712.955318231,         
    12559.038152982,         
     2379.164473572,         
    -6127.655450557,         
      131.541961686,         
    35371.887265976,         
     1066.495477190,         
    17654.780539750,         
       36.027866677,         
      515.463871093,         
      148.078724426,         
      309.278322656,         
      -39.617508346,         
      412.371096874,         
    11403.676995575,         
    13521.751441591,         
   -65147.619767937,         
    10177.257679534,         
     5767.611978898,         
    11087.285125918,         
    14945.316173554,         
     5429.879468239,         
    28766.924424484,         
    11856.218651625,         
    -5481.254918868,         
    22003.914634870,         
     6134.997125565,         
      625.670192312,         
     3496.032826134,         
     6489.261398429,         
  -143571.324284214,         
    12043.574281889,         
    12416.588502848,         
     4686.889407707,         
     7342.457780181,         
     3634.621024518,         
    18635.928454536,         
     -323.505416657,         
    25158.601719765,         
      846.082834751,         
   -12569.674818332,         
     6179.983075773,         
    83467.156352816,         
    10344.295065386,         
    18422.629359098,         
     1265.567478626,         
      -18.159247265,         
    11190.377900137,         
     9623.688276691,         
     5739.157790895,         
    16858.482532933,         
    72140.628666286,         
    17267.268201691,         
     4907.302050146,         
       14.977853527,         
      224.344795702,         
    20426.571092422,         
     5749.452731634,         
     5757.317038160,         
     6702.560493867,         
     6055.549660552,         
     5959.570433334,         
    12562.628581634,         
    39302.096962196,         
    12132.439962106,         
    12029.347187887,         
    -7477.522860216,         
    11609.862544012,         
    17253.041107690,         
    -4535.059436924,         
    21954.157609398,         
       17.252277143,         
    18052.929543158,         
    13517.870106233,         
    -5756.908003246,         
    10557.594160824,         
    20199.094959633,         
    11933.367960670,         
    10454.501386605,         
    15671.081759407,         
      138.517496871,         
     9388.005909415,         
     5749.861766548,         
     6915.859589305,         
    24072.921469776,         
     -640.877607382,         
    12592.450019783,         
    12146.667056108,         
     9779.108676125,         
     6132.028180148,         
     6268.848755990,         
    17996.031168222,         
     -533.214083444,         
     6065.844601290,         
       24.298513841,         
    -2388.894020449,         
     3097.883822726,         
      709.933048357,         
    13095.842665077,         
     6073.708907816,         
      742.990060533,         
    29088.811415985,         
    12359.966151546,         
    10440.274292604,         
      838.969287750,         
    16496.361396202,         
    20597.243963041,         
     6148.010769956,         
     5636.065016677,         
     6080.822454817,         
     -377.373607916,         
     2118.763860378,         
     5867.523359379,         
  -226858.238553767,         
   167283.761587465,         
    28237.233459389,         
    12345.739057544,         
    19800.945956225,         
    43232.306658416,         
    18875.525869774,         
    -1823.175188677,         
      109.945688789,         
      813.550283960,         
   316428.228673312,         
     5756.566278634,         
     5750.203491159,         
    12489.885628707,         
     6303.851245484,         
     1581.959348283,         
     5642.198242609,         
      -70.849445304,         
     6287.008003254,         
      533.623118358,         
    -6279.485421340,         
   -10988.808157535,         
     -227.526189440,         
      415.552490612,         
    29296.615389579,         
    66567.485864652,         
    -3646.350377354,         
    13119.721102825,         
     -209.366942175,         
    25934.124331089,         
     4061.219215394,         
     5113.487598583,         
     1478.866574064,         
    11823.161639450,         
    10770.893256262,         
     6546.159773364,         
       70.328180442,         
    20995.392966449,         
    10660.686935042,         
    33019.021112205,         
    -4933.208440333,         
     -135.625325010,         
    23141.558382925,         
     6144.558353121,         
     6084.003848555,         
    17782.732072784,         
    16460.333529525,         
     5905.702242076,         
      227.476132789,         
    16737.577236597,         
     6805.653268085,         
    11919.140866668,         
      127.471796607,         
     6286.666278643,         
      153.778810485,         
    16723.350142595,         
    11720.068865232,         
     5237.921013804,         
     6709.674040867,         
     4487.817406270,         
     -664.756045130,         
     5127.714692584,         
     6254.626662524,         
    47162.516354635,         
    11080.171578918,         
     -348.924420448,         
      151.047669843,         
     6197.248551160,         
      146.594251718,         
    -5331.357443741,         
       95.979227218,         
    -6418.140930027,         
    -6525.804453965,         
    11293.470674356,         
    -5729.506447149,         
      210.117701700,         
     6066.595360816,         
    18451.078546566,         
    11300.584221356,         
    10027.903195729,         
     4274.518310832,         
     6072.958148291,         
    -7668.637425143,         
    -6245.048177356,         
    -5888.449964932,         
     -543.918059096,         
     9683.594581116,         
     6219.339951688,         
    22743.409379516,         
     1692.165669502,         
     5657.405657679,         
      728.762966531,         
       52.596639600,         
       65.220371012,         
     5881.403728234,         
   163096.180360983,         
    12341.806904281,         
    16627.370915377,         
     1368.660252845,         
     6211.263196841,         
     5792.741760812,         
      -77.750543984,         
     5341.013788022,         
     6281.591377283,         
    -6277.552925684,         
     -525.758811831,         
     6016.468808270,         
    23539.707386333,         
    -4176.041342449,         
    16062.184526117,         
    83783.548222473,         
     9380.959672717,         
     6205.325306007,         
     2699.734819318,         
     -568.821874027,         
     6321.103522627,         
     6321.208885629,         
     1975.492545856,         
      137.033024162,         
    19402.796952817,         
    22805.735565994,         
    64471.991241142,         
      -85.827298831,         
    13613.804277336,         
     9814.604100291,         
    16097.679950283,         
     2107.034507542,         
    36949.230808424,         
   -12539.853380183,         
    -7875.671863624,         
     4171.425536614,         
     6247.911759770,         
     7330.728427345,         
    51092.726050855,         
     5621.842923210,         
      111.430161497,         
      909.818733055,         
     1790.642637886]   

  fh_phi_a = [
  6.240054195,
  4.296977442,
  6.196904410,
  0.444401603,
  4.021195093,
  5.543113262,
  5.025132748,
  5.198467090,
  5.988822341,
  3.649823730,
  1.422745069,
  2.322313077,
  3.615796498,
  4.349338347,
  2.678271909,
  5.696701824,
  0.520007179,
  2.435898309,
  5.866398759,
  4.103476804,
  3.651837925,
  6.153743485,
  4.773852582,
  4.333987818,
  5.957517795,
  1.890075226,
  4.551585768,
  1.914547226,
  0.873504123,
  1.135934669,
  0.092793886,
  5.984503847,
  2.095377709,
  2.949233637,
  2.654394814,
  4.980931759,
  1.280308748,
  4.167901731,
  1.495846011,
  2.251573730,
  4.839650148,
  0.921573539,
  3.411091093,
  2.869567043,
  3.565975565,
  3.324679049,
  3.981496998,
  6.248866009,
  5.071801441,
  5.720622217,
  1.270837679,
  2.546610123,
  5.069663519,
  4.144987272,
  5.210064075,
  0.749317412,
  3.389610345,
  4.183178762,
  2.404714239,
  5.541473556,
  5.040846034,
  1.770181024,
  1.460726241,
  0.652303414,
  3.735430632,
  1.087136918,
  2.901883301,
  5.952658009,
  3.186129845,
  3.307984806,
  1.331103168,
  4.308933301,
  4.005298270,
  6.145309371,
  2.104551349,
  3.475975097,
  5.971672571,
  2.053414715,
  3.016058075,
  1.737438797,
  2.196567739,
  3.969480770,
  5.717799605,
  2.654125618,
  1.913704550,
  4.734090399,
  5.489005403,
  3.661698944,
  4.863931876,
  0.842715011,
  3.293406547,
  4.870690598,
  1.061816410,
  2.465042647,
  1.942176992,
  3.661486981,
  4.920937029,
  1.229392026,
  3.658444681,
  0.673880395,
  5.294249518,
  0.525733528,
  0.837688810,
  4.194535082,
  3.793235253,
  5.639906583,
  0.114858677,
  2.262081818,
  0.642063318,
  6.018501522,
  3.458654112,
  2.500382359,
  2.547107806,
  0.437078094,
  2.246174308,
  2.512929171,
  5.393136889,
  2.999641028,
  1.275847090,
  1.486539246,
  6.140635794,
  4.557814849,
  0.090675389,
  4.869091389,
  4.691976180,
  3.016173439,
  5.554998314,
  1.475415597,
  1.398784824,
  5.650931916,
  2.181745369,
  5.823319737,
  0.003844094,
  3.690360123,
  1.517189902,
  4.447203799,
  1.158692983,
  0.349250250,
  4.893384368,
  1.473760578,
  5.185878737,
  1.030026325,
  3.918259169,
  6.109659023,
  1.892100742,
  4.320519510,
  4.698203059,
  5.325009315,
  1.942656623,
  5.061810696,
  2.797822767,
  0.531673101,
  4.533471191,
  3.734548088,
  2.605547070,
  3.029030662,
  2.355556099,
  5.240963796,
  5.756641637,
  2.013686814,
  1.089100410,
  0.759188178,
  0.645026535,
  6.215885448,
  4.845297676,
  3.809290043,
  6.179611691,
  4.781719760,
  5.956152284,
  1.264976635,
  4.914231596,
  1.405158503,
  2.752035928,
  5.280820144,
  1.571007057,
  4.203664806,
  1.402101526,
  4.167932508,
  0.545828785,
  5.817167450,
  2.419886601,
  2.732084787,
  2.903477885,
  0.593891500,
  1.166145902,
  0.487355242,
  3.913022880,
  3.063805171,
  3.995764039,
  5.152914309,
  3.626395673,
  1.952049260,
  4.996408389,
  5.503379738,
  4.164913291,
  5.841719038,
  2.769753519,
  0.755008103,
  5.306538209,
  4.713486491,
  2.624866359,
  0.382603541,
  0.882213514,
  1.169483931,
  5.448375984,
  2.656486959,
  1.827624012,
  4.387001801,
  3.042700750,
  3.335519004,
  5.601498297,
  1.088831705,
  3.969902609,
  5.142876744,
  0.022067765,
  1.844913056,
  0.968480906,
  4.923831588,
  4.749245231,
  5.991247817,
  2.189604979,
  6.068719637,
  6.197428148,
  3.280414875,
  2.824848947,
  2.501813417,
  2.236346329,
  2.521257490,
  3.370195967,
  3.876512374,
  4.627122566,
  0.156368499,
  5.151962502,
  5.909225055,
  6.054064447,
  2.839021623,
  3.984225404,
  3.852959484,
  2.300991267,
  4.140083146,
  0.437122327,
  4.026532329,
  4.760293101,
  6.270510511,
  6.039606892,
  5.672617711,
  3.397132627,
  6.249666675,
  2.466427018,
  2.242668890,
  5.864091907,
  2.668309141,
  1.681888780,
  2.424978312,
  4.216492400,
  4.772158039,
  6.224271088,
  3.817285811,
  1.649264690,
  3.945345892,
  4.061673868,
  1.794058369,
  3.053874588,
  5.112133338,
  0.513669325,
  3.210727723,
  2.445597761,
  1.676939306,
  5.487314569,
  1.999707589,
  5.783236356,
  3.022645053,
  1.411223013,
  1.179256434,
  1.758191830,
  3.290589143,
  0.864024298,
  2.094441910,
  0.878372791,
  6.003829241,
  4.309591964,
  1.142348571,
  0.210580917,
  4.274476529,
  5.031351030,
  4.959581597,
  4.788002889,
  1.880103788,
  1.405611197,
  1.082356330,
  4.179989585,
  0.353496295,
  1.583849576,
  3.747376371,
  1.395753179,
  0.566790582,
  0.685827538,
  0.731374317,
  3.710043680,
  1.652901407,
  5.827781531,
  2.541182564,
  2.178850542,
  2.253253037,
  0.578340206,
  3.000297967,
  1.693574249,
  1.997249392,
  5.071820966,
  1.072262823,
  4.700657997,
  1.812320441,
  4.322238614,
  3.033827743,
  3.388716544,
  4.687313233,
  0.877776108,
  0.770299429,
  5.353796034,
  4.075291557,
  5.941207518,
  3.208912203,
  4.007881169,
  2.714333592,
  3.215977013,
  0.710872502,
  4.730108488,
  1.327720272,
  0.898769761,
  0.180044365,
  5.152666276,
  5.655385808,
  2.135396205,
  3.708784168,
  1.733578756,
  5.575209112,
  0.069432392,
  4.369302826,
  5.304829118,
  4.096094132,
  3.069327406,
  6.205311188,
  1.417263408,
  3.409035232,
  0.510922054,
  2.389438934,
  5.384595078,
  3.789392838,
  1.661943545,
  5.684549045,
  3.862942261,
  5.549853589,
  1.016115785,
  4.690702525,
  2.911891613,
  1.048982898,
  1.829362730,
  2.636140084,
  4.127883842,
  4.401165650,
  4.175658539,
  4.601102551,
  0.506364778,
  2.032195842,
  4.694756586,
  3.832703118,
  3.308463427,
  1.716090661,
  5.686865780,
  1.942386641,
  1.999482059,
  1.182807992,
  5.934076062,
  2.169080622,
  5.405515999,
  4.656985514,
  3.327476868,
  1.765430262,
  3.454132746,
  5.020393445,
  0.602182191,
  4.960593133,
  2.596451817,
  3.731990323,
  5.636192701,
  3.600691544,
  0.868220961,
  2.629456641,
  2.862574720,
  1.742882331,
  4.812372643,
  0.027860588,
  0.659721876,
  5.245313000,
  4.317625647,
  4.815297007,
  5.381366880,
  4.728443327,
  1.435132069,
  1.354371923,
  3.369695406,
  2.490902145,
  3.799109588,
  1.284375887,
  0.759969109,
  5.409701889,
  3.366890614,
  3.384104996,
  3.803419985,
  0.939039445,
  0.734117523,
  2.564216078,
  4.517099537,
  0.642049130,
  3.485280663,
  3.217431161,
  4.404359108,
  0.366324650,
  4.298212528,
  5.379518958,
  4.527681115,
  6.109429504,
  5.720092446,
  0.679068671,
  4.881123092,
  0.351407289,
  5.146592349,
  0.520791814,
  0.948516300,
  3.504914846,
  5.577502482,
  2.957128968,
  2.598576764,
  3.985702050,
  0.014730471,
  0.085077024,
  0.708426604,
  3.121576600,
  0.288231904,
  2.797450317,
  2.788904128,
  5.895222200,
  6.096188999,
  2.028195445,
  4.660008502,
  4.066520001,
  2.936315115,
  3.223844306,
  1.638054048,
  5.481603249,
  2.205734493,
  2.440421099,
  2.547496264,
  2.314608466,
  4.539108237,
  4.538074405,
  2.869040566,
  6.056405489,
  2.540635083,
  4.005732868,
  1.959967212,
  3.589026260,
  1.728627253,
  6.072332087,
  3.716133846,
  2.725771122,
  4.033338079,
  3.441347021,
  0.656372122,
  2.791483066,
  1.815323326,
  5.711033677,
  2.812745443,
  1.965746028]

  fh_b = [                 
  102.156724e-6,           
    1.706807e-6,           
    0.269668e-6,           
    0.265919e-6,           
    0.210568e-6,           
    0.077996e-6,           
    0.054764e-6,           
    0.059146e-6,           
    0.034420e-6,           
    0.032088e-6,           
    0.033595e-6,           
    0.029198e-6,           
    0.027764e-6,           
    0.025190e-6,           
    0.022997e-6,           
    0.024976e-6,           
    0.021774e-6,           
    0.017925e-6,           
    0.013794e-6,           
    0.013276e-6,           
    0.011774e-6,           
    0.012869e-6,           
    0.012152e-6,           
    0.011081e-6,           
    0.010143e-6,           
    0.009357e-6,           
    0.010084e-6,           
    0.008587e-6,           
    0.008628e-6,           
    0.008158e-6,           
    0.007746e-6,           
    0.007670e-6,           
    0.007098e-6,           
    0.006180e-6,           
    0.005818e-6,           
    0.004945e-6,           
    0.004774e-6,           
    0.004687e-6,           
    0.006089e-6,           
    0.005975e-6,           
    0.004229e-6,           
    0.005264e-6,           
    0.003049e-6,           
    0.002974e-6,           
    0.003403e-6,           
    0.003030e-6,           
    0.003210e-6,           
    0.003058e-6,           
    0.002589e-6,           
    0.002927e-6,           
    0.002425e-6,           
    0.002656e-6,           
    0.002445e-6,           
    0.002990e-6,           
    0.002890e-6,           
    0.002498e-6,           
    0.001889e-6,           
    0.002567e-6,           
    0.001803e-6,           
    0.001782e-6,           
    0.001694e-6,           
    0.001704e-6,           
    0.001735e-6,           
    0.001643e-6,           
    0.001680e-6,           
    0.002045e-6,           
    0.001458e-6,           
    0.001437e-6,           
    0.001738e-6,           
    0.001367e-6,           
    0.001344e-6,           
    0.001438e-6,           
    0.001257e-6,           
    0.001358e-6,           
    0.001628e-6,           
    0.001169e-6,           
    0.001162e-6,           
    0.001092e-6,           
    0.001008e-6,           
    0.001008e-6,           
    0.000918e-6,           
    0.001011e-6,           
    0.000753e-6,           
    0.000737e-6,           
    0.000694e-6,           
    0.000701e-6,           
    0.000689e-6,           
    0.000700e-6,           
    0.000664e-6,           
    0.000654e-6,           
    0.000788e-6,           
    0.000628e-6,           
    0.000755e-6,           
    0.000628e-6,           
    0.000635e-6,           
    0.000534e-6,           
    0.000543e-6,           
    0.000517e-6,           
    0.000504e-6,           
    0.000485e-6,           
    0.000463e-6,           
    0.000604e-6,           
    0.000443e-6,           
    0.000570e-6,           
    0.000465e-6,           
    0.000424e-6,           
    0.000427e-6,           
    0.000478e-6,           
    0.000414e-6,           
    0.000512e-6,           
    0.000378e-6,           
    0.000402e-6,           
    0.000453e-6,           
    0.000395e-6,           
    0.000371e-6,           
    0.000350e-6,           
    0.000356e-6,           
    0.000344e-6,           
    0.000383e-6,           
    0.000333e-6,           
    0.000340e-6,           
    0.000334e-6,           
    0.000399e-6,           
    0.000314e-6,           
    0.000424e-6,           
    0.000307e-6,           
    0.000329e-6,           
    0.000357e-6,           
    0.000312e-6,           
    0.000301e-6,           
    0.000268e-6,           
    0.000257e-6,           
    0.000290e-6,           
    0.000256e-6,           
    0.000339e-6,           
    0.000283e-6,           
    0.000241e-6,           
    0.000304e-6,           
    0.000259e-6,           
    0.000238e-6,           
    0.000236e-6,           
    0.000296e-6,           
    0.000306e-6,           
    0.000251e-6,           
    0.000290e-6,           
    0.000261e-6,           
    0.000249e-6,           
    0.000213e-6,           
    0.000223e-6,           
    0.000268e-6,           
    0.000209e-6,           
    0.000193e-6,           
    0.000182e-6,           
    0.000184e-6,           
    0.000182e-6,           
    0.000228e-6,           
    0.000166e-6,           
    0.000167e-6,           
    0.000159e-6,           
    0.000154e-6,           
    0.000176e-6,           
    0.000167e-6,           
    0.000153e-6,           
    0.000157e-6,           
    0.000142e-6,           
    0.000152e-6,           
    0.000144e-6,           
    0.000135e-6,           
    0.000134e-6,           
    0.000144e-6,           
    0.000160e-6,           
    0.000133e-6,           
    0.000134e-6,           
    0.000134e-6,           
    0.000128e-6,           
    0.000160e-6,           
    0.000132e-6,           
    0.000122e-6,           
    0.000125e-6,           
    0.000121e-6,           
    0.000136e-6,           
    0.000120e-6,           
    0.000134e-6,           
    0.000137e-6,           
    0.000141e-6,           
    0.000129e-6,           
    0.000116e-6,           
    0.000116e-6,           
    0.000129e-6,           
    0.000113e-6,           
    0.000122e-6,           
    0.000140e-6,           
    0.000108e-6,           
    0.000106e-6,           
    0.000110e-6,           
    0.000115e-6,           
    0.000134e-6,           
    0.000109e-6,           
    0.000102e-6,           
    0.000108e-6,           
    0.000101e-6,           
    0.000103e-6,           
    0.000104e-6,           
    0.000101e-6,           
    0.000100e-6]           

  fh_omega_b = [          
    6283.075849991,        
   12566.151699983,        
     213.299095438,        
     529.690965095,        
      -3.523118349,        
    5223.693919802,        
    1577.343542448,        
      26.298319800,        
    -398.149003408,        
   18849.227549974,        
    5507.553238667,        
    5856.477659115,        
     155.420399434,        
    5746.271337896,        
    -796.298006816,        
    5760.498431898,        
     206.185548437,        
    -775.522611324,        
     426.598190876,        
    6062.663207553,        
   12036.460734888,        
    6076.890301554,        
    1059.381930189,        
      -7.113547001,        
    4694.002954708,        
    5486.777843175,        
     522.577418094,        
   10977.078804699,        
    6275.962302991,        
    -220.412642439,        
    2544.314419883,        
    2146.165416475,        
      74.781598567,        
    -536.804512095,        
    5088.628839767,        
   -6286.598968340,        
    1349.867409659,        
    -242.728603974,        
    1748.016413067,        
   -1194.447010225,        
     951.718406251,        
     553.569402842,        
    5643.178563677,        
    6812.766815086,        
   -2352.866153772,        
     419.484643875,        
      -7.046236698,        
    9437.762934887,        
   12352.852604545,        
    5216.580372801,        
    5230.807466803,        
    3154.687084896,        
   10447.387839604,        
    4690.479836359,        
    5863.591206116,        
    6438.496249426,        
    8031.092263058,        
     801.820931124,        
  -71430.695617928,        
       3.932153263,        
   -4705.732307544,        
   -1592.596013633,        
    5849.364112115,        
    8429.241266467,        
      38.133035638,        
    7084.896781115,        
    4292.330832950,        
      20.355319399,        
    6279.552731642,        
   14143.495242431,        
    7234.794256242,        
   11499.656222793,        
    6836.645252834,        
   11513.883316794,        
    7632.943259650,        
     103.092774219,        
    4164.311989613,        
    6069.776754553,        
   17789.845619785,        
     639.897286314,        
   10213.285546211,        
   -6256.777530192,        
   16730.463689596,        
   11926.254413669,        
    3340.612426700,        
    3894.181829542,        
    -135.065080035,        
   13367.972631107,        
    6040.347246017,        
    5650.292110678,        
    6681.224853400,        
    5333.900241022,        
    -110.206321219,        
    6290.189396992,        
   25132.303399966,        
    5966.683980335,        
    -433.711737877,        
   -1990.745017041,        
    5767.611978898,        
    5753.384884897,        
    7860.419392439,        
     515.463871093,        
   12168.002696575,        
     199.072001436,        
   10969.965257698,        
   -7079.373856808,        
     735.876513532,        
   -6127.655450557,        
   10973.555686350,        
    1589.072895284,        
   10984.192351700,        
   11371.704689758,        
    9917.696874510,        
     149.563197135,        
    5739.157790895,        
   11790.629088659,        
    6133.512652857,        
     412.371096874,        
     955.599741609,        
    6496.374945429,        
    6055.549660552,        
    1066.495477190,        
   11506.769769794,        
   18319.536584880,        
    1052.268383188,        
      63.735898303,        
      29.821438149,        
    6309.374169791,        
   -3738.761430108,        
     309.278322656,        
   12043.574281889,        
   12491.370101415,        
     625.670192312,        
    5429.879468239,        
    3496.032826134,        
    3930.209696220,        
   12528.018664345,        
    4686.889407707,        
   16200.772724501,        
   12139.553509107,        
    6172.869528772,        
   -7058.598461315,        
   10575.406682942,        
   17298.182327326,        
    4732.030627343,        
    5884.926846583,        
    5547.199336460,        
   11712.955318231,        
    4701.116501708,        
    -640.877607382,        
    5636.065016677,        
   10177.257679534,        
    6283.143160294,        
    -227.526189440,        
   -6283.008539689,        
   -6284.056171060,        
    7238.675591600,        
    3097.883822726,        
    -323.505416657,        
   -4136.910433516,        
   12029.347187887,        
   12132.439962106,        
     202.253395174,        
   17267.268201691,        
   83996.847317911,        
   17260.154654690,        
    6084.003848555,        
    5756.566278634,        
    5750.203491159,        
    5326.786694021,        
   11015.106477335,        
    3634.621024518,        
   18073.704938650,        
    1162.474704408,        
    5642.198242609,        
     632.783739313,        
   13916.019109642,        
   14314.168113050,        
   12359.966151546,        
    5749.452731634,        
    -245.831646229,        
    5757.317038160,        
   12146.667056108,        
    6206.809778716,        
   17253.041107690,        
   -7477.522860216,        
    5540.085789459,        
    9779.108676125,        
    5237.921013804,        
    5959.570433334,        
    6282.095528923,        
     -11.045700264,        
   23543.230504682,        
  -12569.674818332,        
    -266.607041722,        
   12559.038152982,        
   -2388.894020449,        
   10440.274292604,        
    -543.918059096,        
   21228.392023546,        
   -4535.059436924,        
      76.266071276,        
     949.175608970,        
   13517.870106233,        
   11933.367960670]        
   
  fh_phi_b = [
  4.249032005,
  4.205904248,
  3.400290479,
  5.836047367,
  6.262738348,
  4.670344204,
  4.534800170,
  1.083044735,
  5.980077351,
  4.162913471,
  5.980162321,
  0.623811863,
  3.745318113,
  2.980330535,
  1.174411803,
  2.467913690,
  3.854787540,
  1.092065955,
  2.699831988,
  5.845801920,
  2.292832062,
  5.333425680,
  6.222874454,
  5.154724984,
  4.044013795,
  3.416081409,
  0.749320262,
  2.777152598,
  4.562060226,
  5.806891533,
  1.603197066,
  3.000200440,
  0.443725817,
  1.302642751,
  4.827723531,
  0.268305170,
  5.808636673,
  5.154890570,
  4.403765209,
  2.583472591,
  0.931172179,
  2.336107252,
  1.362634430,
  1.583012668,
  2.552189886,
  5.286473844,
  1.863796539,
  4.226420633,
  1.991935820,
  2.319951253,
  3.084752833,
  2.487447866,
  2.347139160,
  6.235872050,
  0.095197563,
  2.994779800,
  3.569003717,
  3.425611498,
  2.192295512,
  5.180433689,
  4.641779174,
  3.997097652,
  0.417558428,
  2.180619584,
  4.164529426,
  0.526323854,
  1.356098141,
  3.895439360,
  0.087484036,
  3.987576591,
  0.090454338,
  0.974387904,
  1.509069366,
  0.495572260,
  4.968445721,
  2.838496795,
  3.408387778,
  3.617942651,
  0.286350174,
  1.610762073,
  5.532798067,
  0.661826484,
  3.905030235,
  4.641956361,
  2.111120332,
  2.760823491,
  4.768800780,
  5.760439898,
  1.051215840,
  4.911332503,
  4.699648011,
  5.024608847,
  4.370971253,
  3.660478857,
  4.121051532,
  1.173284524,
  0.345585464,
  5.414571768,
  2.328281115,
  1.685874771,
  5.297703006,
  0.591998446,
  4.830881244,
  3.899190272,
  0.476681802,
  1.112242763,
  1.994214480,
  3.778025483,
  5.441088327,
  0.107123853,
  0.915087231,
  4.107281715,
  1.917490952,
  2.763124165,
  3.112111866,
  0.440639857,
  5.444568842,
  5.676832684,
  5.559734846,
  0.261537984,
  5.975534987,
  2.335063907,
  5.321230910,
  2.313312404,
  1.211961766,
  3.169551388,
  6.106912080,
  4.223760346,
  2.180556645,
  1.499984572,
  2.447520648,
  3.662331761,
  1.272834584,
  1.913426912,
  4.165930011,
  4.325565754,
  3.832324536,
  1.612348468,
  3.470173146,
  1.147977842,
  3.776271728,
  0.460368852,
  0.554749016,
  0.834332510,
  4.759564091,
  0.298259862,
  3.749366406,
  5.415666119,
  2.703203558,
  0.283670793,
  1.238477199,
  1.943251340,
  2.456157599,
  5.888038582,
  0.241332086,
  2.657323816,
  5.930629110,
  5.570955333,
  5.786670700,
  1.517805532,
  3.139266834,
  3.556352289,
  1.463313961,
  1.586837396,
  0.022670115,
  0.708528947,
  5.187075177,
  1.993229262,
  3.457197134,
  6.066193291,
  1.710431974,
  2.836451652,
  5.453106665,
  5.326898811,
  2.511652591,
  5.628785365,
  0.819294053,
  5.677408071,
  5.251984735,
  2.210924603,
  1.646502367,
  3.240883049,
  3.059480037,
  1.867105418,
  2.069217456,
  2.781469314,
  4.281176991,
  3.320925381,
  3.497704076,
  0.983210840,
  2.674938860,
  4.957936982,
  1.390113589,
  0.429631317,
  5.501340197,
  4.691456618,
  0.577313584,
  6.218148717,
  1.477842615,
  2.237753948,
  3.100492232,
  5.594294322,
  5.674287810,
  2.196632348,
  4.056084160]

  fh_c = [  
  4.322990e-6,                   
  0.406495e-6,           
  0.122605e-6,           
  0.019476e-6,           
  0.016916e-6,           
  0.013374e-6,           
  0.008042e-6,           
  0.007824e-6,           
  0.004894e-6,           
  0.004875e-6,           
  0.004416e-6,           
  0.004088e-6,           
  0.004433e-6,           
  0.003277e-6,           
  0.002703e-6,           
  0.003435e-6,           
  0.002618e-6,           
  0.003146e-6,           
  0.002544e-6,           
  0.002218e-6,           
  0.002197e-6,           
  0.002897e-6,           
  0.001766e-6,           
  0.001738e-6,           
  0.001695e-6,           
  0.001584e-6,           
  0.001503e-6,           
  0.001552e-6,           
  0.001370e-6,           
  0.001889e-6,           
  0.001722e-6,           
  0.001124e-6,           
  0.001258e-6,           
  0.000831e-6,           
  0.000767e-6,           
  0.000756e-6,           
  0.000775e-6,           
  0.000597e-6,           
  0.000568e-6,           
  0.000711e-6,           
  0.000499e-6,           
  0.000671e-6,           
  0.000488e-6,           
  0.000621e-6,           
  0.000495e-6,           
  0.000456e-6,           
  0.000451e-6,           
  0.000435e-6,           
  0.000387e-6,           
  0.000547e-6,           
  0.000522e-6,           
  0.000375e-6,           
  0.000421e-6,           
  0.000439e-6,           
  0.000309e-6,           
  0.000347e-6,           
  0.000317e-6,           
  0.000262e-6,           
  0.000248e-6,           
  0.000245e-6,           
  0.000225e-6,           
  0.000214e-6,           
  0.000205e-6,           
  0.000180e-6,           
  0.000229e-6,           
  0.000214e-6,           
  0.000175e-6,           
  0.000209e-6,           
  0.000173e-6,           
  0.000184e-6,           
  0.000227e-6,           
  0.000154e-6,           
  0.000151e-6,           
  0.000197e-6,           
  0.000197e-6,           
  0.000138e-6,           
  0.000149e-6,           
  0.000137e-6,           
  0.000135e-6,           
  0.000139e-6,           
  0.000142e-6,           
  0.000120e-6,           
  0.000131e-6,           
  0.000124e-6,           
  0.000108e-6]           

  fh_omega_c = [         
    6283.075849991,          
       0.000000000,         
   12566.151699983,         
     213.299095438,         
     529.690965095,         
      -3.523118349,         
      26.298319800,         
     155.420399434,         
    5746.271337896,         
    5760.498431898,         
    5223.693919802,         
      -7.113547001,         
   77713.771467920,         
   18849.227549974,         
    6062.663207553,         
    -775.522611324,         
    6076.890301554,         
     206.185548437,         
    1577.343542448,         
    -220.412642439,         
    5856.477659115,         
    5753.384884897,         
     426.598190876,         
    -796.298006816,         
     522.577418094,         
    5507.553238667,         
    -242.728603974,         
    -536.804512095,         
    -398.149003408,         
   -5573.142801634,         
    6069.776754553,         
    1059.381930189,         
     553.569402842,         
     951.718406251,         
    4694.002954708,         
    1349.867409659,         
     -11.045700264,         
    2146.165416475,         
    5216.580372801,         
    1748.016413067,         
   12036.460734888,         
   -1194.447010225,         
    5849.364112115,         
    6438.496249426,         
   -6286.598968340,         
    5230.807466803,         
    5088.628839767,         
    5643.178563677,         
   10977.078804699,         
  161000.685737473,         
    3154.687084896,         
    5486.777843175,         
    5863.591206116,         
    7084.896781115,         
    2544.314419883,         
    4690.479836359,         
     801.820931124,         
     419.484643875,         
    6836.645252834,         
   -1592.596013633,         
    4292.330832950,         
    7234.794256242,         
    5767.611978898,         
   10447.387839604,         
     199.072001436,         
     639.897286314,         
    -433.711737877,         
     515.463871093,         
    6040.347246017,         
    6309.374169791,         
  149854.400134205,         
    8031.092263058,         
    5739.157790895,         
    7632.943259650,         
      74.781598567,         
    6055.549660552,         
   -6127.655450557,         
    3894.181829542,         
    9437.762934887,         
   -2352.866153772,         
    6812.766815086,         
   -4705.732307544,         
  -71430.695617928,         
    6279.552731642,         
   -6256.777530192]         
   
  fh_phi_c = [
  2.642893748,  
  4.712388980,
  2.438140634,
  1.642186981,
  4.510959344,
  1.502210314,
  0.478549024,
  5.254710405,
  4.683210850,
  0.759507698,
  6.028853166,
  0.060926389,
  3.627734103,
  2.327912542,
  1.271941729,
  0.747446224,
  3.633715689,
  5.647874613,
  6.232904270,
  1.309509946,
  2.407212349,
  5.863842246,
  0.754113147,
  2.714942671,
  2.629369842,
  1.341138229,
  0.377699736,
  2.904684667,
  1.265599125,
  4.413514859,
  2.445966339,
  5.041799657,
  3.849557278,
  2.471094709,
  5.363125422,
  1.046195744,
  0.245548001,
  4.543268798,
  4.178853144,
  5.934271972,
  0.624434410,
  4.136047594,
  2.209679987,
  4.518860804,
  1.868201275,
  1.271231591,
  0.084060889,
  3.324456609,
  4.052488477,
  2.841633844,
  2.171979966,
  4.983027306,
  4.546432249,
  0.522967921,
  3.172606705,
  1.479586566,
  3.553088096,
  0.606635550,
  3.014082064,
  5.519526220,
  2.877956536,
  1.605227587,
  0.625804796,
  3.499954526,
  5.632304604,
  5.960227667,
  2.162417992,
  2.322150893,
  2.556183691,
  4.732296790,
  5.385812217,
  5.120720920,
  4.815000443,
  0.222827271,
  3.910456770,
  1.397484253,
  5.333727496,
  4.281749907,
  5.979971885,
  4.715630782,
  0.513330157,
  0.194160689,
  0.000379226,
  2.122264908,
  0.883445696]

  fh_d = [              
  0.143388e-6,          
  0.006671e-6,          
  0.001480e-6,          
  0.000934e-6,          
  0.000795e-6,          
  0.000673e-6,          
  0.000672e-6,          
  0.000389e-6,          
  0.000373e-6,          
  0.000360e-6,          
  0.000316e-6,          
  0.000315e-6,          
  0.000278e-6,          
  0.000238e-6,          
  0.000185e-6,          
  0.000245e-6,          
  0.000180e-6,          
  0.000200e-6,          
  0.000141e-6,          
  0.000104e-6]          

  fh_omega_d = [          
   6283.075849991,         
  12566.151699983,         
    155.420399434,         
    213.299095438,         
    529.690965095,         
   5746.271337896,         
   5760.498431898,         
   -220.412642439,         
   6062.663207553,         
   6076.890301554,         
    -21.340641002,         
   -242.728603974,         
    206.185548437,         
   -536.804512095,         
    522.577418094,         
  18849.227549974,         
    426.598190876,         
    553.569402842,         
   5223.693919802,         
   5856.477659115]         
    
  fh_phi_d = [
  1.131453581,
  0.775148887,
  0.480016880,
  6.144453084,
  2.941595619,
  0.120415406,
  5.317009738,
  3.090323467,
  3.003551964,
  1.918913041,
  5.545798121,
  1.884932563,
  1.266254859,
  4.532664830,
  4.578313856,
  0.587467082,
  5.151178553,
  5.355983739,
  1.336556009,
  4.239842759]
  
  fh_e = [             
  0.003826e-6,         
  0.000303e-6,         
  0.000209e-6]         

  fh_omega_e = [            
   6283.075849991,           
  12566.151699983,           
    155.420399434]               

  fh_phi_e = [
  5.705257275,
  5.407132842,
  1.989815753]    
  
  jd_tdbs = zeros(BigFloat,length(jd_tts))
  for i in 1:length(jd_tts)
           
    tt = (jd_tts[i] - 2451545.0)/365250.0

    dt_a = 0.0
    dt_b = 0.0
    dt_c = 0.0
    dt_d = 0.0
    dt_e = 0.0

    for j in 1:length(fh_a)
      dt_a = dt_a + fh_a[j]*sin(fh_omega_a[j]*tt + fh_phi_a[j])
    end

    for j in 1:length(fh_b)
      dt_b = dt_b + fh_b[j]*sin(fh_omega_b[j]*tt + fh_phi_b[j])
    end
    dt_b = tt*dt_b

    for j in 1:length(fh_c)
      dt_c = dt_c + fh_c[j]*sin(fh_omega_c[j]*tt + fh_phi_c[j])
    end
    dt_c = tt*tt*dt_c

    for j in 1:length(fh_d)
      dt_d = dt_d + fh_d[j]*sin(fh_omega_d[j]*tt + fh_phi_d[j])
    end
    dt_d = tt*tt*tt*dt_d
     
    for j in 1:length(fh_e)
      dt_e = dt_e + fh_e[j]*sin(fh_omega_e[j]*tt + fh_phi_e[j])
    end
    dt_e = tt*tt*tt*tt*dt_e
    
    dt = BigFloat(dt_a + dt_b + dt_c + dt_d + dt_e)     

#   print(1000.0*dt)

    jd_tdbs[i] = jd_tts[i] + dt/BigFloat(86400.0)

    
  end 

  return jd_tdbs
end


#  Apply the delay to obtain TDB scale using kaplan parameters

function include_kaplan_delay(jd_tts)

  jd_tdbs = zeros(BigFloat,length(jd_tts))
  for i in 1:length(jd_tts)
           
    t = (jd_tts[i] - BigFloat(2451545.0))/BigFloat(36525.0)

    dt = BigFloat(0.001657)*sin(BigFloat(628.3076)*t + BigFloat(6.2401))
    dt = dt + BigFloat(0.000022)*sin(BigFloat(575.3385)*t + BigFloat(4.2970))
    dt = dt + BigFloat(0.000014)*sin(BigFloat(1256.6152)*t + BigFloat(6.1969))
    dt = dt + BigFloat(0.000005)*sin(BigFloat(606.9777)*t + BigFloat(4.0212))
    dt = dt + BigFloat(0.000005)*sin(BigFloat(52.9691)*t + BigFloat(0.4444))
    dt = dt + BigFloat(0.000002)*sin(BigFloat(21.3299)*t + BigFloat(5.5431))
    dt = dt + BigFloat(0.000010)*sin(BigFloat(628.3076)*t + BigFloat(4.2490))
 
#   print(1000.0*dt)

    jd_tdbs[i] = jd_tts[i] + dt / BigFloat(86400.0)

    
  end 

  return jd_tdbs
end


# Execution begins here

#  Read the comand line for the arguments
jd_tts =   get_jdtt_args(ARGS)

# Include the delay

if kaplan_flag
  jd_tdbs = include_kaplan_delay(jd_tts)
else
  jd_tdbs = include_fairhead_delay(jd_tts)
end

# Display formatted result with estimate of 64-bit numerical error 

njds = length(jd_tdbs)
if njds < 1
  exit()
end  
print("\n")
least_count_error = BigFloat(0.0001)*BigFloat(86400.0) 
@printf("  Least count accurate to %4.2f nanoseconds \n", least_count_error)
@printf("     JD_TT                  JD_TDB \n")
for i in 1:njds
  @printf("%20.13f  %20.13f \n", jd_tts[i], jd_tdbs[i])
  end
print("\n")

exit()

