#!/usr/local/bin/julia

# fits_stack_median
#
#   Create mean, median, and difference images from a stack of fits images
#
#   Explicit inputs:
#
#     Directory of FITS files to be processed
#
#   Explicit output:
#
#     Median image of the image stack
#
#
#   John Kielkopf (kielkopf@louisville.edu)
#   MIT License
#   Copyright 2021
#   
#   2021-07-20 Version 1.0

# Julia language notes

#   Current documentation
#   See https://docs.julialang.org/en/v1/

#   Very helpful guidance from Wikibooks
#   See https://en.wikibooks.org/wiki/Introducing_Julia

#   Julia High Performance guides optimization
#   See https://juliahighperformance.com/

#   See https://juliaastro.github.io/dev/index.html

#   FITSIO is high level interface to CFITSIO 
#   In REPL => Pkg.add("FITSIO")
#   In code => using  FITSIO does not require "FITSIO." before the functions
#   In code => import FITSIO requires "FITSIO."
#   See https://juliaastro.github.io/FITSIO.jl/stable/ for documentation

#   CFITSIO functions are thin wrappers around the CFITSIO C library routines
#   See https://juliaastro.github.io/CFITSIO.jl/dev/

#   Warning:  Julia is column major like Matlab and Fortran.
#   It is always the last dimension that is stored first 
#   with the values for that dimension one after the other in memory.
#   To optimize fast access to consecutively stored data
#   with a 2-dimensional array in for loop,  operate on it 
#   to vary the last dimension most slowly and sequentially
#   get the values in that "column" one after the other. In this
#   example x[i, j] is modified with "i" changing for each fixed "j" in 
#   the innermost loop. Here "i" runs down the column "j" of the 2x2 matrix. 
#
#      for j in 1:size(x, 2) 
#        for i in 1:size(x, 1) 
#          s = s + x[i, j] ^ 2   
#          x[i, j] = s           
#        end                   
#      end                    

#   Optimized for speed by using slices with images as first index
#   Version 1.0k with 4 threads runs 2x faster than Python Numpy

#   When using threads declare this environment variable
#
#     export "JULIA_NUM_THREADS=4"
#
#   for the number of threads equal to the number of CPUs available   


using FITSIO               # For FITS management
using Dates                # Date and time
using Statistics           # For median
using Base.Threads         # For multiprocessing

# Input a stack of image data with images as the first index
# Return a median image through the stack

function stack_median(images)
  (d, w, h) = size(images)
  median_image = zeros(w, h)
  @threads for j in 1:h
    for i in 1:w
      median_image[i,j] = median(images[:,i,j])
    end   
  end    
  return median_image
end


# Input a stack of image data with images as the first index
# Return a median image through the stack

function stack_mean(images)
  (d, w, h) = size(images)
  mean_image = zeros(w, h)
  @threads for j in 1:h
    for i in 1:w
      mean_image[i,j] = mean(images[:,i,j])
    end   
  end    
  return mean_image
end


# Accept list of file names as command line arguments
# Return an array of file names

function get_files_by_name()

  nfiles = length(ARGS)
  nimages = nfiles - 1

  if nimages < 3
    println(" ")
    println("Usage: infile1.fits infile2.fits infile3.fits ... ")
    println(" ")
    println("Create a median image from a time- or z-axis stack\n ")
    exit()
  else
    infiles = ARGS[1:nfiles]
  end

  return infiles
end  
    

# Accept directory as command line argument
# Return an array of file names

function get_files_by_directory()

  if length(ARGS) == 1
    directory = ARGS[1]
  else
    directory = "."
  end
    
  infiles = readdir(directory)
  nfiles = length(infiles)

  if nfiles < 3
    println(" ")
    println("Usage:  fits_stack_mean_median.jl [directory] ")
    println(" ")
    println("Creates mean, median, and difference images from from a time- or z-axis stack\n ")
    println("Defaults to the current working directory")
    println("Requires 3 or more files \n")
    exit()
  end

  for i in 1:nfiles
    infiles[i] = directory*"/"*infiles[i]
  end
    
  return infiles
end  
    

# Build and return an image stack from an array of FITS file names

function build_image_stack(infiles)
  
  d = length(infiles)
  
  f = FITS(infiles[1])
  (w, h) = size(f[1])
  close(f)

  # Initialize the image stack using the size of the first image
  images = zeros(d, w, h)
   
  # Sequentially add images from the files to the images stack
  
  for i in 1:d   

    f = FITS(infiles[i])
    image = read(f[1])
    (width, height) = size(f[1])
    close(f)
    if (w != width) & (h != height)
      println("Warning: image file " * infiles[i] * " has a anomalous size")
      exit()
    end

    # Assign the new image into the initialized stack
    # such that each pixel has its sliced data sequentia in memory
    
    # Multiprocess with a for loop 
    # rather than single process  with images[i, :, :] .= image
    
    @threads for k in 1:h
      for j in 1:w
        images[i, j, k] = image[j,k]
      end   
    end    
   
  end  
   
  # Return the image stack
  
  return images

end  


# Process the images to find and export the median image and a header

function process_images()

  println("Acquiring image file names")
  infiles = get_files_by_directory()
  nimages = length(infiles)

  println("Building the image stack in memory")
  images = @time build_image_stack(infiles)

  println("Computing the median image through the stack \n")
  median_image = @time stack_median(images)

  println("Computing the mean image through the stack \n")
  mean_image = @time stack_mean(images)

  println("Computing the difference (mean minus median) image  \n")
  difference_image =  mean_image .- median_image

  # Use the header of the first file for the stack
  
  f = FITS(infiles[1])
  first_header = read_header(f[1])
  close(f)
  
  # Update the header date stamp
 
  file_time_str = string(DateTime(now()))
  first_header["DATE"] = file_time_str
    
  return mean_image, median_image, difference_image, first_header, nimages

end


# Process the requested images

mean_image, median_image, difference_image, header, nimages = process_images()


# Add descriptive comments and write the files to disk
# Append a COMMENT based on headers.jl in FITSIO source
# Needed because updating by key replaces previous COMMENT values
# FITS allows multiple comments
# The header has three arrays and a map that is a dictionary

push!(header.keys, "COMMENT")
comment_string = "Median of " * string(nimages) * " images"
push!(header.values, "")
push!(header.comments, comment_string)
header.map["COMMENT"] = length(header.keys)
median_file = "median.fits"
f = FITS(median_file, "w")
write(f, median_image; header=header)
close(f)

comment_string = "Mean of " * string(nimages) * " images"
set_comment!(header, "COMMENT", comment_string )
mean_file = "mean.fits"
f = FITS(mean_file, "w")
write(f, mean_image; header=header)
close(f)

comment_string = "Difference of mean and median images"
set_comment!(header, "COMMENT", comment_string )
difference_file = "difference.fits"
f = FITS(difference_file, "w")
write(f, difference_image; header=header)
close(f)

println("\n")
println("The mean, median, and difference images  have been saved \n")

exit()

